/*
 * This software is contributed or developed by KYOCERA Corporation.
 * (C) 2015 KYOCERA Corporation
 * (C) 2016 KYOCERA Corporation
 * (C) 2017 KYOCERA Corporation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 and
 * only version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */
#define pr_fmt(fmt)	"BMS: %s: " fmt, __func__

#include <linux/module.h>
#include <linux/power_supply.h>
#include <oem-bms.h>
#include <linux/rtc.h>
#include <linux/wakelock.h>

#define VBATT_SAMPLES			5
#define POWER_OFF_SOC			0
#define POWER_OFF_THRESHOLD_UV		(3300 * 1000)
#define VBATT_INITIAL_UV			(4200 * 1000)

static int vbatt_current_uv = VBATT_INITIAL_UV;
static int vbatt_buf_index = 0;
static int vbatt_buf[VBATT_SAMPLES] ={
	VBATT_INITIAL_UV, VBATT_INITIAL_UV, VBATT_INITIAL_UV, VBATT_INITIAL_UV, VBATT_INITIAL_UV
};

#define OEM_FAKE_CAPACITY		1
#define LOWTEMP_THRESH			(-100)
#define LOWTEMP_CHECK_RETURN_TIME_MS	(120 * 1000)
#define BOOT_POWER_OFF_THRESHOLD_UV	(3100 * 1000)
static int oem_lowtemp_flag = false;
static int power_off_thresh_uv = POWER_OFF_THRESHOLD_UV;
static atomic_t is_oem_lowtemp_check_initialized = ATOMIC_INIT(0);
static struct delayed_work oem_release_of_lowtemp_work;
static struct wake_lock lowtemp_boot_wake_lock;


#define DISCHARGING_FILTER_TIME_S	60
#define CHARGING_FILTER_TIME_S		60
#define CORRECT_DISCHG_CALC_SOC		50

static atomic_t is_oem_bms_initialized = ATOMIC_INIT(0);
static int pre_result_soc = 0;
static unsigned long filter_base_time = 0;

static int oem_bms_get_current_time(unsigned long *now_tm_sec)
{
	struct rtc_time tm;
	struct rtc_device *rtc;
	int rc;

	rtc = rtc_class_open(CONFIG_RTC_HCTOSYS_DEVICE);
	if (rtc == NULL) {
		pr_err("%s: unable to open rtc device (%s)\n",
			__FILE__, CONFIG_RTC_HCTOSYS_DEVICE);
		return -EINVAL;
	}

	rc = rtc_read_time(rtc, &tm);
	if (rc) {
		pr_err("Error reading rtc device (%s) : %d\n",
			CONFIG_RTC_HCTOSYS_DEVICE, rc);
		goto close_time;
	}

	rc = rtc_valid_tm(&tm);
	if (rc) {
		pr_err("Invalid RTC time (%s): %d\n",
			CONFIG_RTC_HCTOSYS_DEVICE, rc);
		goto close_time;
	}
	rtc_tm_to_time(&tm, now_tm_sec);

close_time:
	rtc_class_close(rtc);
	return rc;
}

static int oem_bms_get_batt_prop(enum power_supply_property psp)
{
	union power_supply_propval ret = {0,};
	static struct power_supply *psy;

	psy = power_supply_get_by_name("battery");
	if (psy) {
		/* if battery has been registered, use the status property */
		psy->get_property(psy, psp, &ret);
		return ret.intval;
	}

	/* Default to false if the battery power supply is not registered. */
	pr_err("battery power supply is not registered\n");
	return -EINVAL;
}

static int oem_bms_get_average(int *buf, int num)
{
	int ave, cnt;
	int max = 0, min = 0, sum = 0;

	max = buf[0];
	min = buf[0];

	for (cnt = 0; cnt < num; cnt++) {
		if (max < buf[cnt]) {
			max = buf[cnt];
		} else if (min > buf[cnt]) {
			min = buf[cnt];
		}
		sum += buf[cnt];
	}

	if (num < 3) {
		ave = sum / num;
	} else {
		ave = (sum - max - min) / (num - 2);
	}

	pr_debug("ave:%d sum:%d max:%d min:%d num:%d\n",
		ave, sum, max, min, num);

	return ave;
}

static void oem_bms_update_vbatt_current(void)
{
	int vbatt_now_uv = oem_bms_get_batt_prop(POWER_SUPPLY_PROP_VOLTAGE_NOW);

	if (vbatt_now_uv == -EINVAL) {
		pr_err("invalid value vbatt_now_uv:%d\n", vbatt_now_uv);
		return;
	}
	vbatt_buf_index++;

	if (vbatt_buf_index == VBATT_SAMPLES) {
		vbatt_buf_index = 0;
	}
	vbatt_buf[vbatt_buf_index] = vbatt_now_uv;
	vbatt_current_uv = oem_bms_get_average(vbatt_buf, VBATT_SAMPLES);

	pr_debug("vbatt_buf [0]=%d [1]=%d [2]=%d [3]=%d [4]=%d vbatt_buf_index:%d\n",
		vbatt_buf[0], vbatt_buf[1], vbatt_buf[2], vbatt_buf[3], vbatt_buf[4], vbatt_buf_index);
}

static void oem_release_of_lowtemp_check_work(struct work_struct *work)
{
	power_off_thresh_uv = POWER_OFF_THRESHOLD_UV;
	oem_lowtemp_flag = false;
	pr_info("PowerOff thresh = %duv.lowtemp_flag : %d\n", power_off_thresh_uv, oem_lowtemp_flag);
	wake_unlock(&lowtemp_boot_wake_lock);
	pr_info("unlock lowtemp_boot_wake_lock\n");
}

static void oem_bms_check_boot_lowtemp(void)
{
	int bat_temp = oem_bms_get_batt_prop(POWER_SUPPLY_PROP_TEMP);

	if (bat_temp == -EINVAL) {
		pr_err("invalid value bat_temp:%d\n", bat_temp);
		return;
	}

	if (bat_temp <= LOWTEMP_THRESH) {
		power_off_thresh_uv = BOOT_POWER_OFF_THRESHOLD_UV;
		oem_lowtemp_flag = true;
		wake_lock(&lowtemp_boot_wake_lock);
		pr_info("wake lock lowtemp_boot_wake_lock\n");
		schedule_delayed_work(&oem_release_of_lowtemp_work, msecs_to_jiffies(LOWTEMP_CHECK_RETURN_TIME_MS));
		pr_info("LowTemp : %d. PowerOff thresh = %duv. lowtemp_flag : %d\n", bat_temp, power_off_thresh_uv, oem_lowtemp_flag);
	}
	pr_info("Finished checking LowTempStatus.\n");
	atomic_set(&is_oem_lowtemp_check_initialized, 1);
	return;
}

static int oem_bms_check_low_vbatt(int soc, int battery_status)
{
	int result_soc = soc;

	if(!atomic_read(&is_oem_lowtemp_check_initialized)) {
		oem_bms_check_boot_lowtemp();
	}

	if(vbatt_current_uv <= power_off_thresh_uv) {
		result_soc = POWER_OFF_SOC;
		pr_err("set auto power off soc:%d vbatt_uv:%d\n", soc, vbatt_current_uv);
	} else if (oem_lowtemp_flag && (result_soc == POWER_OFF_SOC)) {
		result_soc = OEM_FAKE_CAPACITY;
		pr_info("LowTemp Flag is true.Skip notifying 0.\n");
	}
	pr_debug("result_soc:%d vbatt_current_uv:%d\n", result_soc, vbatt_current_uv);
	return result_soc;
}

int oem_get_lowtemp_flag_status(void)
{
	return oem_lowtemp_flag;
}
EXPORT_SYMBOL(oem_get_lowtemp_flag_status);

int oem_bms_correct_calc_soc(int calc_soc, int last_soc)
{
	unsigned long now_time = filter_base_time;
	int elapsed_time;
	int filter_check_time = DISCHARGING_FILTER_TIME_S;
	int correct_calc_soc = calc_soc;
	int delta_soc = 0;

	if (!atomic_read(&is_oem_bms_initialized)) {
		pr_err("uninitialized\n");
		return correct_calc_soc;
	}

	if (calc_soc <= CORRECT_DISCHG_CALC_SOC) {
		return correct_calc_soc;
	}

	if (pre_result_soc > calc_soc) {
		delta_soc = pre_result_soc - calc_soc;
		oem_bms_get_current_time(&now_time);
		elapsed_time = now_time - filter_base_time;

		if (elapsed_time >= filter_check_time) {
			delta_soc -= (elapsed_time / filter_check_time);
			if (delta_soc < 0) {
				delta_soc = 0;
			}
			filter_base_time = now_time;

			pr_info("timer expiration delta_soc:%d minus:%d elapsed_time:%d filter_base_time:%d\n",
				delta_soc, elapsed_time / filter_check_time, elapsed_time, (int)filter_base_time);
		}
		correct_calc_soc += delta_soc;

		pr_debug("delta_soc:%d pre_result_soc:%d calc_soc:%d elapsed_time:%d filter_base_time:%d filter_check_time:%d\n",
			delta_soc, pre_result_soc, calc_soc, elapsed_time, (int)filter_base_time, filter_check_time);
	}

	return correct_calc_soc;
}
EXPORT_SYMBOL(oem_bms_correct_calc_soc);

int oem_bms_correct_soc(int in_soc)
{
	unsigned long now_time = filter_base_time;
	int elapsed_time;
	int filter_check_time = CHARGING_FILTER_TIME_S;
	int result_soc = in_soc;
	int battery_status = oem_bms_get_batt_prop(POWER_SUPPLY_PROP_STATUS);

	if (battery_status == -EINVAL) {
		pr_err("invalid value battery_status:%d\n", battery_status);
		return result_soc;
	}

	if ((battery_status == POWER_SUPPLY_STATUS_CHARGING) &&
		(result_soc > pre_result_soc)) {
		oem_bms_get_current_time(&now_time);
		elapsed_time = now_time - filter_base_time;

		if (elapsed_time >= filter_check_time) {
			result_soc = pre_result_soc + 1;
			filter_base_time = now_time;

			pr_info("timer expiration in_soc:%d result_soc:%d  pre_result_soc:%d elapsed_time:%d filter_base_time:%d\n",
				in_soc, result_soc,  pre_result_soc, elapsed_time, (int)filter_base_time);
		} else {
			result_soc = pre_result_soc;

			pr_debug("timer not expiration in_soc:%d result_soc:%d  pre_result_soc:%d elapsed_time:%d filter_base_time:%d\n",
				in_soc, result_soc,  pre_result_soc, elapsed_time, (int)filter_base_time);
		}
	}

	oem_bms_update_vbatt_current();

	result_soc = oem_bms_check_low_vbatt(result_soc, battery_status);

	pr_debug("pre_result_soc:%d result_soc:%d in_soc:%d battery_status:%d\n",
		pre_result_soc, result_soc, in_soc, battery_status);

	pre_result_soc = result_soc;

	return result_soc;
}
EXPORT_SYMBOL(oem_bms_correct_soc);

void oem_bms_init(int calc_soc, int last_soc)
{
	oem_bms_get_current_time(&filter_base_time);

	if (last_soc == -EINVAL) {
		pre_result_soc = calc_soc;
	} else {
		pre_result_soc = last_soc;
	}

	wake_lock_init(&lowtemp_boot_wake_lock, WAKE_LOCK_SUSPEND, "lowtemp_boot");

	INIT_DELAYED_WORK(&oem_release_of_lowtemp_work, oem_release_of_lowtemp_check_work);

	atomic_set(&is_oem_bms_initialized, 1);
	pr_info("initialized pre_result_soc:%d calc_soc:%d last_soc:%d\n", pre_result_soc, calc_soc, last_soc);
}
EXPORT_SYMBOL(oem_bms_init);

